from django.core.exceptions import MultipleObjectsReturned
from django.core.validators import (
    MinLengthValidator,
    RegexValidator,
    MinValueValidator,
    MaxValueValidator,
)
from django.db import models
from django.utils.translation import gettext_lazy as _

from .core import BaseModel
from .mixins import (
    VRPaymentResultMixin,
    VRPaymentResultDetails,
    VRPayApiResponseMixin,
    VRPaymentCardMixin,
    VRPaymentMerchantMixin,
)
from .webhooks import VRPaymentWebhook


from ..fields import VRPaymentDecimalField, JSONField
from ..managers import VRPaymentAPIResponseManger, VRPaymentBasicPaymentManager
from ..utils.transaction_status import (
    check_transaction_successful,
    check_transaction_pending,
    check_transaction_rejected,
)


class AbstractVRPaymentResponse(
    VRPayApiResponseMixin,
    VRPaymentResultMixin,
    VRPaymentResultDetails,
    VRPaymentCardMixin,
    VRPaymentMerchantMixin,
    BaseModel,
):
    """
    all Payment Response Parameters
    from https://vr-pay-ecommerce.docs.oppwa.com/reference/parameters#response-params

    """

    vr_pay_id = models.CharField(
        "ID",
        blank=True,
        help_text="The VR Pay identifier of this object",
        max_length=48,
        null=True,  # blank/null True b/c the request could have been invalid
    )
    reference_id = models.CharField(
        "ReferenceID",
        blank=True,
        help_text="In case of referenced payment (e.g., Capture or Refund), this fields included to see which payment was referenced.",
        max_length=32,
        null=True,
        validators=[RegexValidator(regex=r"[a-zA-Z0-9]{32}")],
    )
    payment_brand = models.CharField(
        "Payment brand",
        blank=True,
        help_text="The payment brand of the request.",
        max_length=32,
        null=True,
        validators=[RegexValidator(regex=r"[a-zA-Z0-9_] {1,32}")],
    )
    amount = VRPaymentDecimalField(
        "Amount", blank=True, help_text="The amount of the request", null=True,
    )
    currency = models.CharField(
        "Currency",
        blank=False,
        default="EUR",
        help_text="The ISO 4217 currency code of the payment request's amount",
        max_length=3,
        null=True,
    )
    # Note: according to documentation descriptor max-length is 127, but depending on config, vr payment might add data to it, before sending it back.
    descriptor = models.CharField(
        "Descriptor",
        blank=True,
        help_text="The descriptor of the request.",
        max_length=255,
        null=True,
    )
    risk_score = models.IntegerField(
        "Risk score",
        blank=True,
        help_text="Returns the score of the executed transaction risk checks. The value is a number from -99999 to +99999. Can be returned both for standalone risk requests and payment requests that include risk checks.",
        validators=[MinValueValidator(-99999), MaxValueValidator(99999)],
        null=True,
    )
    merchant_transaction_id = models.CharField(
        "Merchant transaction ID",
        blank=False,
        help_text="Merchant-provided reference number, should be unique for your transactions. "
        "Some receivers require this ID. This identifier is often used for reconciliation.",
        max_length=255,
        validators=[MinLengthValidator(8)],
    )
    other = JSONField(
        "Other",
        blank=True,
        help_text="The response can also contain each of the data structures listed above, such as 'customer' and 'billingAddress'.",
        null=True,
    )

    objects = VRPaymentAPIResponseManger()

    class Meta:
        abstract = True
        indexes = [
            models.Index(fields=["vr_pay_id"]),
            models.Index(fields=["reference_id"]),
        ]

    def __str__(self):
        return f"{self._meta.model_name}[{self.id}] - {self.http_status_code}"

    @property
    def is_successful(self) -> bool:
        return check_transaction_successful(self.result_code)

    @property
    def is_pending(self) -> bool:
        return check_transaction_pending(self.result_code)

    @property
    def is_rejected(self) -> bool:
        return check_transaction_rejected(self.result_code)


class VRPaymentBasicPayment(BaseModel):
    """
    implementation of Basic Payment
    https://vr-pay-ecommerce.docs.oppwa.com/reference/parameters#basic
    """

    PREAUTHORIZATION = "PA"
    DEBIT = "DB"
    CREDIT = "CD"
    CAPTURE = "CP"
    REVERSAL = "RV"
    REFUND = "RF"
    PAYMENT_TYPE = [
        (PREAUTHORIZATION, _("Preauthorization")),
        (DEBIT, _("Debit")),
        (CREDIT, _("Credit")),
        (CAPTURE, _("Capture")),
        (REVERSAL, _("Reversal")),
        (REFUND, _("Refund")),
    ]
    entity_id = models.CharField(
        "Entity ID",
        blank=False,
        null=False,
        help_text="The entity required to authorize the request. "
        "This should be the channel entity identifier. "
        "In case channel dispatching is activated then it should be the merchant entity identifier.",
        max_length=32,
    )
    amount = VRPaymentDecimalField(
        "Amount",
        help_text="Indicates the amount of the payment request. "
        "The dot is used as decimal separator. "
        "The amount is the only amount value which is processing relevant. "
        "All other amount declarations like taxAmount or shipping.cost are already included.",
    )
    tax_amount = VRPaymentDecimalField(
        "Tax amount", help_text="Indicates the tax amount of the payment request."
    )
    currency = models.CharField(
        "Currency",
        blank=False,
        default="EUR",
        help_text="The ISO 4217 currency code of the payment request's amount",
        max_length=3,
    )
    payment_brand = models.CharField(
        "Payment brand",
        blank=True,
        help_text="The brand specifies the method of payment for the request. "
        "This is optional if you want to use brand detection for credit cards, if not then it is mandatory.",
        max_length=32,
        null=True,
        validators=[RegexValidator(regex=r"[a-zA-Z0-9_]{1,32}")],
    )
    payment_type = models.CharField(
        "Payment type",
        blank=False,
        choices=PAYMENT_TYPE,
        help_text="The payment type for the request. See docs for supported types",
        max_length=2,
    )
    # Note: according to documentation descriptor max-length is 127, but depending on config, vr payment might add data to it, before sending it back.
    descriptor = models.CharField(
        "Descriptor",
        blank=True,
        help_text="Can be used to populate all or part of the Merchant Name descriptor, "
        "which often appears on the first line of the shopper's statement. "
        "The full use of this field depends on the Merchant Account configuration. "
        "NOTE: merchant.name can override any data sent in this field.",
        max_length=255,
        null=True,
    )
    merchant_transaction_id = models.CharField(
        "Merchant transaction ID",
        blank=False,
        help_text="Merchant-provided reference number, should be unique for your transactions. "
        "Some receivers require this ID. This identifier is often used for reconciliation.",
        max_length=255,
        unique=True,
        validators=[MinLengthValidator(8)],
    )
    merchant_invoice_id = models.CharField(
        "Merchant invoice ID",
        blank=True,
        help_text="Merchant-provided invoice number, should be unique for your transactions. "
        "This identifier is not sent onwards.",
        max_length=255,
        null=True,
        unique=True,
        validators=[MinLengthValidator(8)],
    )
    merchant_memo = models.CharField(
        "Merchant memo",
        blank=True,
        help_text="Merchant-provided additional information. The information provided is not transaction processing relevant. "
        "It will appear in reporting only.",
        max_length=255,
        null=True,
        validators=[MinLengthValidator(8)],
    )
    transaction_category = models.CharField(
        "Transaction category",
        blank=True,
        help_text="The category of the transaction. See docs for possible values:",
        max_length=32,
        null=True,
    )
    sandbox = models.BooleanField(
        "Sandbox", help_text="Determine if this transaction uses the Test or Live host"
    )
    resource_path = models.CharField(
        "VR Payment Resource Path",
        blank=True,
        default="",
        help_text="VR Payment resource path can. Can be used to get the status",
        max_length=512,
        null=True,
    )
    payment_id = models.CharField(
        "VR Payment ID",
        blank=True,
        default="",
        help_text="The identifier of the payment request that can be used to reference the payment later",
        max_length=32,
        null=True,
        validators=[RegexValidator(r"[a-zA-Z0-9]{32}")],
    )
    objects = VRPaymentBasicPaymentManager()

    class Meta:
        verbose_name = "VR Payment Basic Payment Checkout"
        verbose_name_plural = "VR Payment Basic Checkouts"
        indexes = [
            models.Index(fields=["merchant_transaction_id"]),
            models.Index(fields=["merchant_invoice_id"]),
        ]

    def __str__(self):
        return f"id: {self.id}, transaction_id: {self.merchant_transaction_id} ({self.amount} {self.currency})"

    @property
    def checkout_id(self):
        return self.checkout_response.vr_pay_id


class VRPaymentBasicPaymentStatusResponse(AbstractVRPaymentResponse):
    basic_payment = models.ForeignKey(
        VRPaymentBasicPayment,
        on_delete=models.CASCADE,
        related_name="payment_responses",
        verbose_name=VRPaymentBasicPayment._meta.verbose_name_plural,
    )

    class Meta:
        verbose_name = "VR Payment Payment Response"
        verbose_name_plural = "VR Payment Payment Responses"


class VRPaymentCheckoutResponse(AbstractVRPaymentResponse):
    basic_payment = models.OneToOneField(
        VRPaymentBasicPayment,
        on_delete=models.CASCADE,
        related_name="checkout_response",
        verbose_name=VRPaymentBasicPayment._meta.verbose_name,
    )

    class Meta:
        verbose_name = "VR Payment Checkout Reponse"
        verbose_name_plural = "VR Payment Checkout Reponses"


class VRPaymentWebhookPaymentPayload(AbstractVRPaymentResponse):
    basic_payment = models.ForeignKey(
        VRPaymentBasicPayment,
        on_delete=models.CASCADE,
        related_name="webhook_responses",
        verbose_name=VRPaymentBasicPayment._meta.verbose_name_plural,
    )
    webhook = models.OneToOneField(
        VRPaymentWebhook,
        on_delete=models.CASCADE,
        related_name="payment_payload",
        verbose_name="VR Payment Webhook Payload",
    )

    class Meta:
        verbose_name = "VR Payment Webhook Payload"
        verbose_name_plural = "VR Payment Webhook Payloads"



import requests
from requests import Response

from .checkout import CheckOutWrapper
from .transaction import TransactionWrapper
from .. import settings


class VRPaymentWrapper(TransactionWrapper, CheckOutWrapper):
    bearer_token = settings.VR_PAYMENT_BEARER_TOKEN
    entity_id = settings.VR_PAYMENT_ENTITY_ID
    sandbox = settings.VR_PAYMENT_SANDBOX
    url = (
        settings.VR_PAYMENT_TEST_URL
        if settings.VR_PAYMENT_SANDBOX
        else settings.VR_PAYMENT_LIVE_URL
    )

    def __init__(
        self, bearer_token: str = None, entity_id: str = None, sandbox: bool = None
    ) -> None:
        self.bearer_token = bearer_token if bearer_token else self.bearer_token
        self.entity_id = entity_id if entity_id else self.entity_id
        self.sandbox = sandbox if sandbox is not None else self.sandbox
        if self.sandbox:
            self.url = f"{settings.VR_PAYMENT_TEST_URL}"
        elif sandbox is not None:
            self.url = f"{settings.VR_PAYMENT_LIVE_URL}"
        super().__init__()

    def _call_api(
        self,
        url_append: str,
        method: str,
        data: dict = None,
        headers: dict = {},
        connect_timeout=2,
        read_timeout=10,
    ) -> Response:
        headers.update({"Authorization": f"Bearer {self.bearer_token}"})
        call_url = self.url + url_append
        try:
            if method is "POST":
                response = requests.post(
                    url=call_url,
                    headers=headers,
                    data=data,
                    timeout=(connect_timeout, read_timeout),
                )
            elif method is "GET":
                response = requests.get(
                    url=call_url,
                    headers=headers,
                    timeout=(connect_timeout, read_timeout),
                )
            else:
                raise NotImplementedError(f"method '{method}# is not supported")
        except requests.RequestException as e:
            raise e
        return response



# Generated by Django 3.1.3 on 2020-11-16 20:21

from ..fields import JSONField

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_vr_payment.fields


class Migration(migrations.Migration):

    dependencies = [
        ('django_vr_payment', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='VRPaymentWebhook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('last_modified', models.DateTimeField(auto_now=True, verbose_name='Last modified')),
                ('raw_headers', JSONField(help_text='response header as json', verbose_name='Headers')),
                ('webhook_type', models.CharField(help_text='The webhook type. Can be PAYMENT, REGISTRATION, RISK (and others in test…)', max_length=64, verbose_name='Webhook type')),
                ('webhook_action', models.CharField(blank=True, help_text='The webhook action. Only applies to REGISTRATION type (and others in test…)', max_length=64, null=True, verbose_name='Webhook action')),
                ('decrypted_body', JSONField(help_text='The decrypted request.body in JSON', verbose_name='Decrypted body')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VRPaymentWebhookPaymentPayload',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('last_modified', models.DateTimeField(auto_now=True, verbose_name='Last modified')),
                ('http_status_code', models.IntegerField(help_text='response status code', verbose_name='HTTP status code')),
                ('url', models.URLField(help_text='request url', verbose_name='URL')),
                ('raw_headers', JSONField(help_text='response header as json', verbose_name='Headers')),
                ('raw_content', JSONField(help_text='response content as json', verbose_name='Content')),
                ('build_number', models.CharField(blank=True, help_text='Useful for support purposes.', max_length=255, null=True, verbose_name='API response build number')),
                ('ndc', models.CharField(blank=True, help_text='An internal unique identifier for the request.', max_length=65, null=True, verbose_name='ndc')),
                ('result_code', models.CharField(blank=True, help_text='Status Code from VR Payment. see content for description', max_length=11, null=True, verbose_name='VR Payment status code')),
                ('result_description', models.CharField(blank=True, help_text="A textual description explaining the result.code's meaning.", max_length=255, null=True, verbose_name='Result Description')),
                ('result_avs_response', models.CharField(blank=True, help_text='Contains the AVS response returned by the acquirer. It may include one the following result:A = Address does match, zip code does not matchZ = Address does not match, zip code does matchN = Address and zip code do not matchU = Technical or logical error. AVS cannot be applied on card or address (not UK or US issuer), issuer is not available, etc.F = Address and Postal Code Matches', max_length=1, null=True, verbose_name='Result avs response')),
                ('result_cvv_response', models.CharField(blank=True, help_text='Contains the CVV response returned by the acquirer.', max_length=1, null=True, verbose_name='Result cvv response')),
                ('result_details', JSONField(blank=True, help_text='A container for name value pair used for enriching the response with bank-specific response details. I.e. the actual parameters used within resultDetails are bank-specific.', null=True, verbose_name='Result details')),
                ('result_details_acquirer_response', JSONField(blank=True, help_text='Represents the acquirer original response code retrieved from the acquirer directly.', null=True, verbose_name='Result details - Acquirer response')),
                ('card_bin', models.CharField(blank=True, help_text='The first six digits of the card.number', max_length=6, null=True, verbose_name='Card bin')),
                ('card_holder', models.CharField(blank=True, help_text='Holder of the credit card account', max_length=6, null=True, verbose_name='Card holder')),
                ('card_expiry_month', models.CharField(blank=True, help_text='The expiry month of the card', max_length=2, null=True, verbose_name='Card expiry month')),
                ('card_expiry_year', models.CharField(blank=True, help_text='The expiry year of the card', max_length=4, null=True, verbose_name='Card expiry year')),
                ('merchant_bank_account_holder', models.CharField(blank=True, help_text="Holder of the merchant's bank account", max_length=128, null=True, verbose_name='Merchant bank account holder')),
                ('merchant_bank_account_nummber', models.CharField(blank=True, help_text="The account number of the merchant's bank account. (IBAN for SEPA accounts)", max_length=64, null=True, verbose_name='Merchant bank account nummber')),
                ('merchant_bank_account_bic', models.CharField(blank=True, help_text="The BIC (Bank Identifier Code (SWIFT)) number of the merchant's bank account.", max_length=11, null=True, verbose_name='Merchant bank account bic')),
                ('merchant_bank_account_country', models.CharField(blank=True, help_text="The country code of the merchant's bank account (ISO 3166-1).", max_length=2, null=True, verbose_name='Merchant bank account country')),
                ('vr_pay_id', models.CharField(blank=True, help_text='The VR Pay identifier of this object', max_length=48, null=True, verbose_name='ID')),
                ('reference_id', models.CharField(blank=True, help_text='In case of referenced payment (e.g., Capture or Refund), this fields included to see which payment was referenced.', max_length=32, null=True, validators=[django.core.validators.RegexValidator(regex='[a-zA-Z0-9]{32}')], verbose_name='ReferenceID')),
                ('payment_brand', models.CharField(blank=True, help_text='The payment brand of the request.', max_length=32, null=True, validators=[django.core.validators.RegexValidator(regex='[a-zA-Z0-9_] {1,32}')], verbose_name='Payment brand')),
                ('amount', django_vr_payment.fields.VRPaymentDecimalField(blank=True, decimal_places=2, help_text='The amount of the request', max_digits=12, null=True, verbose_name='Amount')),
                ('currency', models.CharField(default='EUR', help_text="The ISO 4217 currency code of the payment request's amount", max_length=3, null=True, verbose_name='Currency')),
                ('descriptor', models.CharField(blank=True, help_text='The descriptor of the request.', max_length=127, null=True, verbose_name='Descriptor')),
                ('risk_score', models.IntegerField(blank=True, help_text='Returns the score of the executed transaction risk checks. The value is a number from -99999 to +99999. Can be returned both for standalone risk requests and payment requests that include risk checks.', null=True, validators=[django.core.validators.MinValueValidator(-99999), django.core.validators.MaxValueValidator(99999)], verbose_name='Risk score')),
                ('merchant_transaction_id', models.CharField(help_text='Merchant-provided reference number, should be unique for your transactions. Some receivers require this ID. This identifier is often used for reconciliation.', max_length=255, validators=[django.core.validators.MinLengthValidator(8)], verbose_name='Merchant transaction ID')),
                ('other', JSONField(blank=True, help_text="The response can also contain each of the data structures listed above, such as 'customer' and 'billingAddress'.", null=True, verbose_name='Other')),
            ],
            options={
                'verbose_name': 'VR Payment Webhook Payload',
                'verbose_name_plural': 'VR Payment Webhook Payloads',
            },
        ),
        migrations.RemoveIndex(
            model_name='vrpaymentbasicpayment',
            name='vr_payment__merchan_9521ba_idx',
        ),
        migrations.RemoveIndex(
            model_name='vrpaymentbasicpayment',
            name='vr_payment__merchan_44f0b9_idx',
        ),
        migrations.AddField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='merchant_transaction_id',
            field=models.CharField(default='replace_me', help_text='Merchant-provided reference number, should be unique for your transactions. Some receivers require this ID. This identifier is often used for reconciliation.', max_length=255, validators=[django.core.validators.MinLengthValidator(8)], verbose_name='Merchant transaction ID'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='vrpaymentcheckoutresponse',
            name='merchant_transaction_id',
            field=models.CharField(default='replace_me', help_text='Merchant-provided reference number, should be unique for your transactions. Some receivers require this ID. This identifier is often used for reconciliation.', max_length=255, validators=[django.core.validators.MinLengthValidator(8)], verbose_name='Merchant transaction ID'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='other',
            field=JSONField(blank=True, help_text="The response can also contain each of the data structures listed above, such as 'customer' and 'billingAddress'.", null=True, verbose_name='Other'),
        ),
        migrations.AlterField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='raw_content',
            field=JSONField(help_text='response content as json', verbose_name='Content'),
        ),
        migrations.AlterField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='raw_headers',
            field=JSONField(help_text='response header as json', verbose_name='Headers'),
        ),
        migrations.AlterField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='result_details',
            field=JSONField(blank=True, help_text='A container for name value pair used for enriching the response with bank-specific response details. I.e. the actual parameters used within resultDetails are bank-specific.', null=True, verbose_name='Result details'),
        ),
        migrations.AlterField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='result_details_acquirer_response',
            field=JSONField(blank=True, help_text='Represents the acquirer original response code retrieved from the acquirer directly.', null=True, verbose_name='Result details - Acquirer response'),
        ),
        migrations.AlterField(
            model_name='vrpaymentcheckoutresponse',
            name='other',
            field=JSONField(blank=True, help_text="The response can also contain each of the data structures listed above, such as 'customer' and 'billingAddress'.", null=True, verbose_name='Other'),
        ),
        migrations.AlterField(
            model_name='vrpaymentcheckoutresponse',
            name='raw_content',
            field=JSONField(help_text='response content as json', verbose_name='Content'),
        ),
        migrations.AlterField(
            model_name='vrpaymentcheckoutresponse',
            name='raw_headers',
            field=JSONField(help_text='response header as json', verbose_name='Headers'),
        ),
        migrations.AlterField(
            model_name='vrpaymentcheckoutresponse',
            name='result_details',
            field=JSONField(blank=True, help_text='A container for name value pair used for enriching the response with bank-specific response details. I.e. the actual parameters used within resultDetails are bank-specific.', null=True, verbose_name='Result details'),
        ),
        migrations.AlterField(
            model_name='vrpaymentcheckoutresponse',
            name='result_details_acquirer_response',
            field=JSONField(blank=True, help_text='Represents the acquirer original response code retrieved from the acquirer directly.', null=True, verbose_name='Result details - Acquirer response'),
        ),
        migrations.AddIndex(
            model_name='vrpaymentbasicpayment',
            index=models.Index(fields=['merchant_transaction_id'], name='django_vr_p_merchan_fe8f32_idx'),
        ),
        migrations.AddIndex(
            model_name='vrpaymentbasicpayment',
            index=models.Index(fields=['merchant_invoice_id'], name='django_vr_p_merchan_106324_idx'),
        ),
        migrations.AddField(
            model_name='vrpaymentwebhookpaymentpayload',
            name='basic_payment',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='webhook_responses', to='django_vr_payment.vrpaymentbasicpayment', verbose_name='VR Payment Basic Checkouts'),
        ),
        migrations.AddField(
            model_name='vrpaymentwebhookpaymentpayload',
            name='webhook',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='payment_payload', to='django_vr_payment.vrpaymentwebhook', verbose_name='VR Payment Webhook Payload'),
        ),
    ]



# Generated by Django 2.2.28 on 2025-10-10 09:25

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_vr_payment', '0002_add_webhooks'),
    ]

    operations = [
        migrations.AlterField(
            model_name='vrpaymentbasicpayment',
            name='descriptor',
            field=models.CharField(blank=True, help_text="Can be used to populate all or part of the Merchant Name descriptor, which often appears on the first line of the shopper's statement. The full use of this field depends on the Merchant Account configuration. NOTE: merchant.name can override any data sent in this field.", max_length=255, null=True, verbose_name='Descriptor'),
        ),
        migrations.AlterField(
            model_name='vrpaymentbasicpaymentstatusresponse',
            name='descriptor',
            field=models.CharField(blank=True, help_text='The descriptor of the request.', max_length=255, null=True, verbose_name='Descriptor'),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _

from ..fields import JSONField
from ..managers import VRPaymentWebhookManager

from .core import BaseModel


class VRPaymentWebhook(BaseModel):
    raw_headers = JSONField("Headers", help_text="response header as json")
    webhook_type = models.CharField(
        "Webhook type",
        blank=False,
        help_text="The webhook type. Can be PAYMENT, REGISTRATION, RISK (and others in test…)",
        max_length=64,  # don't actually know; test environment != docs…
        null=False,
    )
    webhook_action = models.CharField(
        "Webhook action",
        blank=True,
        help_text="The webhook action. Only applies to REGISTRATION type (and others in test…)",
        max_length=64,  # don't actually know; test environment != docs…
        null=True,
    )
    decrypted_body = JSONField(
        "Decrypted body",
        blank=False,
        help_text="The decrypted request.body in JSON",
        null=False,
    )

    objects = VRPaymentWebhookManager()

    def save(
        self, force_insert=False, force_update=False, using=None, update_fields=None
    ):
        super().save(force_insert, force_update, using, update_fields)
        if self.webhook_type == "payment":
            if not self.payment_payload:
                from django_vr_payment.models.payment import (
                    VRPaymentWebhookPaymentPayload,
                )

                VRPaymentWebhookPaymentPayload.objects.create_from_response(
                    self.decrypted_payload
                )



from decimal import Decimal

import requests
from django.utils import timezone
from requests import Response

from ..models import (
    VRPaymentBasicPaymentStatusResponse,
    VRPaymentBasicPayment,
    VRPaymentCheckoutResponse,
)


class CheckOutWrapper(object):
    def create_checkout(
        self,
        amount: Decimal,
        payment_type: str,
        merchant_transaction_id: str,
        currency: str = "EUR",
        tax_amount: Decimal = Decimal(0.00),
        payment_brand: str = None,
        descriptor: str = None,
        merchant_invoice_id: str = None,
        merchant_memo: str = None,
        transaction_category: str = None,
        **kwargs,
    ) -> Response:
        """
        create a checkout. see VR Payment documentation for all available fields
        :param amount: complete payable amount as Decimal
        :param payment_type: one of VRPaymentBasicPayment.PAYMENT_TYPE
        :param merchant_transaction_id: Merchant-provided reference number, must be unique for your transactions
        :param currency: The currency code of the payment request's amount
        :param tax_amount: Tax amount as Decimal
        :param payment_brand: (optional) The brand specifies the method of payment for the request
        :param descriptor: (optional) Can be used to populate all or part of the Merchant Name descriptor, which often appears on the first line of the shopper's statement. Max 127 chars.
        :param merchant_invoice_id: (optional) Merchant-provided invoice number, should be unique for your transactions.
        :param merchant_memo: (optional) Merchant-provided additional information
        :param transaction_category: (optional) The category of the transaction. See VR Payment docs for possible values
        :param kwargs: (optional) any additional information you want to provide. See VR Payment docs for possible kwargs. NOTE: these values will not be saved in the VRPaymentBasicPayment object
        """
        data = {
            "entityId": self.entity_id,
            "amount": amount,
            "currency": currency,
            "paymentType": payment_type,
            "taxAmount": tax_amount,
            "paymentBrand": payment_brand,
            "descriptor": descriptor[:127] if descriptor else None,
            "merchantTransactionId": merchant_transaction_id,
            "merchantInvoiceId": merchant_invoice_id,
            "merchantMemo": merchant_memo[:255] if merchant_memo else None,
            "transactionCategory": transaction_category,
        }
        if kwargs:
            # for some reasons "billing.city" is not meant as json but as string, so we need to convert kwargs
            for kwarg in kwargs:
                for element in kwargs.get(kwarg):
                    data.update({kwarg + "." + element: kwargs.get(kwarg).get(element)})
        response = self._call_api("/v1/checkouts", "POST", data=data)
        basic_payment = VRPaymentBasicPayment.objects.create(
            entity_id=self.entity_id,
            amount=amount,
            currency=currency,
            payment_type=payment_type,
            tax_amount=tax_amount,
            payment_brand=payment_brand,
            descriptor=descriptor[:255] if descriptor else None,
            merchant_transaction_id=merchant_transaction_id,
            merchant_invoice_id=merchant_invoice_id,
            merchant_memo=merchant_memo[:255] if merchant_memo else None,
            transaction_category=transaction_category,
            sandbox=self.sandbox,
        )
        VRPaymentCheckoutResponse.objects.create_from_response(
            response, basic_payment=basic_payment
        )
        return basic_payment

    def get_checkout_status(
        self, basic_payment: VRPaymentBasicPayment
    ) -> VRPaymentBasicPaymentStatusResponse:
        url = f"{basic_payment.resource_path}?entityId={self.entity_id}"
        try:
            assert basic_payment.created_at > (
                timezone.now() - timezone.timedelta(minutes=30)
            )
            response = self._call_api(url, "GET")
            response.raise_for_status()
        except (AssertionError, requests.HTTPError):
            # checkout status can only be retrieved once for 30 min
            # or initial checkout status might already be handled but missing;
            # -> try with merchant_transaction_id
            if basic_payment.merchant_transaction_id:
                return self.get_transaction_by_merchant_transaction_id(basic_payment)
        return VRPaymentBasicPaymentStatusResponse.objects.create_from_response(
            response, basic_payment=basic_payment
        )



import os
import binascii
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend


def decrypt_webhook(
    config_key: str, Initialization_vector: str, auth_tag: str, http_body: str
):

    # Convert data to process
    key = binascii.unhexlify(config_key)
    iv = binascii.unhexlify(Initialization_vector)
    auth_tag = binascii.unhexlify(auth_tag)
    cipher_text = binascii.unhexlify(http_body)

    # Prepare decryption
    decryptor = Cipher(
        algorithms.AES(key), modes.GCM(iv, auth_tag), backend=default_backend()
    ).decryptor()

    # Decrypt
    result = decryptor.update(cipher_text) + decryptor.finalize()
    return result



from django.db import models

try:
    # Django 3.1
    from django.db.models import JSONField as BaseJSONField
except ImportError:
    from django.contrib.postgres.fields import JSONField as BaseJSONField


class APIResponseMixin(models.Model):
    http_status_code = models.IntegerField(
        "HTTP status code",
        help_text="response status code"
    )
    url = models.URLField(
        "URL",
        help_text="request url")
    raw_headers = BaseJSONField(
        "Headers",
        help_text="response header as json")
    raw_content = BaseJSONField(
        "Content",
        help_text="response content as json")

    class Meta:
        abstract = True


class VRPayApiResponseMixin(APIResponseMixin):
    build_number = models.CharField(
        "API response build number",
        blank=True,
        help_text="Useful for support purposes.",
        max_length=255,
        null=True,
    )
    ndc = models.CharField(
        "ndc",
        blank=True,
        help_text="An internal unique identifier for the request.",
        max_length=65,
        null=True,
    )

    class Meta:
        abstract = True


class VRPaymentResultMixin(models.Model):
    result_code = models.CharField(
        "VR Payment status code",
        blank=True,
        help_text="Status Code from VR Payment. see content for description",
        max_length=11,  # xxx.yyy.zzz
        null=True,
    )  # https://vr-pay-ecommerce.docs.oppwa.com/reference/resultCodes
    result_description = models.CharField(
        "Result Description",
        blank=True,
        help_text="A textual description explaining the result.code's meaning.",
        max_length=255,
        null=True,
    )
    result_avs_response = models.CharField(
        "Result avs response",
        blank=True,
        help_text="Contains the AVS response returned by the acquirer. It may include one the following result:"
                  "A = Address does match, zip code does not match"
                  "Z = Address does not match, zip code does match"
                  "N = Address and zip code do not match"
                  "U = Technical or logical error. AVS cannot be applied on card or address (not UK or US issuer), issuer is not available, etc."
                  "F = Address and Postal Code Matches",
        max_length=1,
        null=True,
    )
    result_cvv_response = models.CharField(
        "Result cvv response",
        blank=True,
        help_text="Contains the CVV response returned by the acquirer.",
        max_length=1,
        null=True,
    )

    class Meta:
        abstract = True


class VRPaymentResultDetails(models.Model):
    result_details = BaseJSONField(
        "Result details",
        blank=True,
        help_text="A container for name value pair used for enriching the response with bank-specific response details. I.e. the actual parameters used within resultDetails are bank-specific.",
        null=True,
    )
    result_details_acquirer_response = BaseJSONField(
        "Result details - Acquirer response",
        blank=True,
        help_text="Represents the acquirer original response code retrieved from the acquirer directly.",
        null=True,
    )

    class Meta:
        abstract = True


class VRPaymentCardMixin(models.Model):
    card_bin = models.CharField(
        "Card bin",
        blank=True,
        help_text="The first six digits of the card.number",
        max_length=6,
        null=True,
    )
    card_holder = models.CharField(
        "Card holder",
        blank=True,
        help_text="Holder of the credit card account",
        max_length=6,
        null=True,
    )
    card_expiry_month = models.CharField(
        "Card expiry month",
        blank=True,
        help_text="The expiry month of the card",
        max_length=2,
        null=True,
    )
    card_expiry_year = models.CharField(
        "Card expiry year",
        blank=True,
        help_text="The expiry year of the card",
        max_length=4,
        null=True,
    )

    class Meta:
        abstract = True


class VRPaymentMerchantMixin(models.Model):
    merchant_bank_account_holder = models.CharField(
        "Merchant bank account holder",
        blank=True,
        help_text="Holder of the merchant's bank account",
        max_length=128,
        null=True,
    )
    merchant_bank_account_nummber = models.CharField(
        "Merchant bank account nummber",
        blank=True,
        help_text="The account number of the merchant's bank account. (IBAN for SEPA accounts)",
        max_length=64,
        null=True,
    )
    merchant_bank_account_bic = models.CharField(
        "Merchant bank account bic",
        blank=True,
        help_text="The BIC (Bank Identifier Code (SWIFT)) number of the merchant's bank account.",
        max_length=11,
        null=True,
    )
    merchant_bank_account_country = models.CharField(
        "Merchant bank account country",
        blank=True,
        help_text="The country code of the merchant's bank account (ISO 3166-1).",
        max_length=2,
        null=True,
    )

    class Meta:
        abstract = True



from .payment import (
    VRPaymentBasicPayment,
    VRPaymentBasicPaymentStatusResponse,
    VRPaymentCheckoutResponse,
    VRPaymentWebhookPaymentPayload,
)
from .webhooks import VRPaymentWebhook

__all__ = [
    "VRPaymentBasicPayment",
    "VRPaymentBasicPaymentStatusResponse",
    "VRPaymentCheckoutResponse",
    "VRPaymentWebhookPaymentPayload",
    "VRPaymentWebhook",
]
